Program
  ps1exec;

{$MODE OBJFPC}
{$APPTYPE GUI}
{$H+}

{$IFDEF x86} {$R ps1exec_x86.res} {$ENDIF}
{$IFDEF x64} {$R ps1exec_x64.res} {$ENDIF}

Uses
  CmdTail,
  Error32,
  Exec,
  PathsFiles,
  Registry,
  GetOpts,
  Windows;

Const
  APP_TITLE = 'ps1exec';
  PS_SHELLID_SUBKEY: PChar = 'SOFTWARE\Microsoft\PowerShell\1\ShellIds\Microsoft.PowerShell';

Var
  Opt: Char;
  WindowState: LongInt;
  StartDir, WindowTitle, Executable, ScriptName, ScriptParams, Parameters, TempScriptName: String;
  Elevate, Wait, Quiet, KeepOpen, LoadProfile, NonInteractive, MTA, UseTempScript: Boolean;
  F: Text;

Procedure Usage;
  Var
    Msg: String;
  Begin
  Msg := APP_TITLE + ' - (C) 2009-2018 by Bill Stewart (bstewart@iname.com)' + #10 +
    #10 +
    'Runs a PowerShell script.' + #10 +
    #10 +
    'Usage: ' + APP_TITLE + ' [options] "<scriptfile>" [-- param [...]]' + #10 +
    #10 +
    'Options are:' + #10 +
    #10 +
    '-s <n>'      + #9 + 'Set the initial window state (see below)' + #10 +
    '-d "<path>"' + #10 +
                    #9 + 'Set starting directory (ignored with -e)' + #10 +
    '-e'          + #9 + 'Request elevation (i.e., ''Run as administrator'')' + #10 +
    '-w'          + #9 + 'Wait for script to complete and return PowerShell exit code' + #10 +
    '-q'          + #9 + 'Run quietly (no error dialogs)' + #10 +
    '-t "<title>"' + #10 +
                    #9 + 'Specify a console window title (implies -w)' + #10 +
    '-k'          + #9 + 'Keep console window after script finishes (implies -w)' + #10 +
    '-n'          + #9 + 'Run script non-interactively (-NonInteractive)' + #10 +
    '-p'          + #9 + 'Load PowerShell profile (omit -NoProfile)' + #10 +
    '-m'          + #9 + 'Run using a multi-threaded apartment (-Mta)' + #10 +
    '--'          + #9 + 'Everything after ''--'' is script parameters' + #10 +
    #10 +
    'Values for -s: 0 = hidden, 1 = normal + active, 2 = minimize + active, 3 = maximize + active, 4 = normal + not active, 7 = minimized + not active';
  MessageBox(0, PChar(Msg), APP_TITLE, MB_ICONINFORMATION);
  End;

Procedure Complain(Const Prefix: String; Const ErrCode: DWORD);
  Begin
  If Not Quiet Then
    MessageBox(0, PChar(Prefix + #10 + ErrToStr(ErrCode)), APP_TITLE, MB_ICONEXCLAMATION);
  End;

Function StrToL(Const S: String; Var L: LongInt): Boolean;
  Var
    Code: Word;
  Begin
  Val(S, L, Code);
  StrToL := Code = 0;
  End;

// Returns the path and filename for powershell.exe
Function GetPSPath: String;
  Var
    Path: String;
  Begin
  If GetRegistryStringValue(HKEY_LOCAL_MACHINE, PS_SHELLID_SUBKEY, 'Path', Path) = 0 Then
    Result := Path
  Else
    Result := '';
  End;

Begin
  If (ParamCount = 0) Or (ParamStr(1) = '/?') Then
    Begin
    Usage;
    Exit;
    End;

  OptErr := False;  // Don't complain about invalid options
  WindowState := SW_SHOWNORMAL;
  StartDir := GetCurrDir;
  WindowTitle := '';
  Elevate := False;
  Wait := False;
  Quiet := False;
  KeepOpen := False;
  LoadProfile := False;
  NonInteractive := False;
  MTA := False;
  UseTempScript := False;

  Repeat
    Opt := GetOpt('S:s:D:d:EeWwQqT:t:KkNnPpMm');
    Case Opt Of
      'S', 's':
        Begin
        If Not StrToL(OptArg, WindowState) Then
          Begin
          Complain('-s parameter requires a numeric value.', ERROR_BAD_ARGUMENTS);
          ExitCode := ERROR_BAD_ARGUMENTS;
          Exit;
          End;
        End;
      'D', 'd':
        Begin
        StartDir := OptArg;
        If StartDir = '' Then
          Begin
          Complain('-d parameter requires a directory name.', ERROR_BAD_ARGUMENTS);
          ExitCode := ERROR_BAD_ARGUMENTS;
          Exit;
          End;
        End;
      'E', 'e':
        Begin
        Elevate := True;
        End;
      'W', 'w':
        Begin
        Wait := True;
        End;
      'Q', 'q':
        Begin
        Quiet := True;
        End;
      'T', 't':
        Begin
        WindowTitle := OptArg;
        If WindowTitle = '' Then
          Begin
          Complain('-t parameter requires a window title.', ERROR_BAD_ARGUMENTS);
          ExitCode := ERROR_BAD_ARGUMENTS;
          Exit;
          End;
        UseTempScript := True;
        End;
      'K', 'k':
        Begin
        KeepOpen := True;
        UseTempScript := True;
        End;
      'N', 'n':
        Begin
        NonInteractive := True;
        End;
      'P', 'p':
        Begin
        LoadProfile := True;
        End;
      'M', 'm':
        Begin
        MTA := True;
        End;
    End; //Case
  Until Opt = EndOfOptions;

  Executable := GetPSPath;
  If Not FileExists(Executable) Then
    Begin
    Complain('Can''t find powershell.exe in registry', ERROR_PATH_NOT_FOUND);
    ExitCode := ERROR_PATH_NOT_FOUND;
    Exit;
    End;

  // Script name is missing
  If ParamStr(OptInd) = '' Then
    Begin
    If Not Quiet Then
      Usage;
    ExitCode := ERROR_BAD_ARGUMENTS;
    Exit;
    End;

  // Script file not found
  ScriptName := ParamStr(OptInd);
  If Not FileExists(ScriptName) Then
    Begin
    Complain('Script file not found - ' + ScriptName, ERROR_FILE_NOT_FOUND);
    ExitCode := ERROR_FILE_NOT_FOUND;
    Exit;
    End;

  ScriptParams := GetCommandTail(OptInd + 1);

  If UseTempScript Then
    Begin
    Wait := True;
    ExitCode := GetTempFilenameEx('', 'script', '.ps1', True, TempScriptName);
    If ExitCode <> 0 Then
      Begin
      Complain('Unable to create temporary script file - ' + TempScriptName, ExitCode);
      Exit;
      End;
    Assign(F, TempScriptName);
    Rewrite(F);
    If WindowTitle <> '' Then
      WriteLn(F, '$Host.UI.RawUI.WindowTitle = ''', WindowTitle, '''');
    Write(F, '& ''', ScriptName, '''');
    If ScriptParams <> '' Then
      Write(F, ' ', ScriptParams);
    WriteLn(F);
    If KeepOpen Then
      WriteLn(F, 'Read-Host ''Press ENTER to continue''');
    WriteLn(F, 'exit $LASTEXITCODE');
    Close(F);
    ScriptName := TempScriptName;
    ScriptParams := '';
    End;

  Parameters := '';

  If Not LoadProfile Then
    Parameters := Parameters + '-NoProfile ';

  If NonInteractive Then
    Parameters := Parameters + '-NonInteractive ';

  If MTA Then
    Parameters := Parameters + '-Mta ';

  Parameters := Parameters + '-ExecutionPolicy Bypass -File "' + ScriptName + '"';
  If Length(ScriptParams) > 0 then
    Parameters := Parameters + ' ' + ScriptParams;

  ExitCode := RunExecutable(Executable, Parameters, StartDir, WindowState, Wait, Quiet, Elevate);

  If UseTempScript Then
    Begin
    {$I-}
    Erase(F);
    {$I+}
    If IOResult <> 0 Then
      Begin
      Complain('Error deleting temporary script file - ' + TempScriptName, IOResult);
      ExitCode := IOResult;
      End;
    End;

End.
